@@ -1,6 +1,6 @@
 ;;; ada-core.el --- core facilities for ada-mode -*- lexical-binding:t -*-
 
-;; Copyright (C) 1994, 1995, 1997 - 2017, 2019  Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995, 1997 - 2017, 2019 - 2020  Free Software Foundation, Inc.
 ;;
 ;; Author: Stephen Leake <stephen_leake@member.fsf.org>
 ;; Maintainer: Stephen Leake <stephen_leake@member.fsf.org>
@@ -48,20 +48,11 @@
   "Default Ada compiler; can be overridden in project files.
 Values defined by compiler packages.")
 
-(defvar ada-mode-hook nil
-  "List of functions to call when Ada mode is invoked.
-This hook is executed after `ada-mode' is fully loaded, but
-before file local variables are processed.")
-
 (defvar ada-syntax-propertize-hook nil
   "Hook run from `ada-syntax-propertize'.
 Called by `syntax-propertize', which is called by font-lock in
 `after-change-functions'.")
 
-(defun ada-in-based-numeric-literal-p ()
-  "Return t if point is after a prefix of a based numeric literal."
-  (looking-back "\\([0-9]+#[0-9a-fA-F_]+\\)" (line-beginning-position)))
-
 (defun ada-declarative-region-start-p (cache)
   "Return t if cache is a keyword starting a declarative region."
   (memq (wisi-cache-token cache) '(DECLARE IS PRIVATE))
@@ -475,13 +466,6 @@ extend a with_clause to include CHILD-NAME  .	"
 (defconst ada-xref-known-tools '(gpr_query gnat)
   "Supported xref tools")
 
-(defcustom ada-xref-full-path nil
-  "If t, cross-references show the full path to source files; if
-nil, only the file name."
-  :group 'ada
-  :type 'boolean
-  :safe #'booleanp)
-
 (defun ada-make-subprogram-body ()
   "Convert subprogram specification after point into a subprogram body stub."
   (interactive)
