@@ -3,7 +3,7 @@
 ;; gpr-query supports Ada and any gcc language that supports the
 ;; AdaCore -fdump-xref switch (which includes C, C++).
 ;;
-;; Copyright (C) 2013 - 2019  Free Software Foundation, Inc.
+;; Copyright (C) 2013 - 2020  Free Software Foundation, Inc.
 
 ;; Author: Stephen Leake <stephen_leake@member.fsf.org>
 ;; Maintainer: Stephen Leake <stephen_leake@member.fsf.org>
@@ -50,6 +50,10 @@ Value must be alist where each element is \"<name>=<value>\""
   ;; which must be set for all projects on the system.
   :type 'string)
 
+(defconst gpr-query-protocol-version "2"
+  "Defines data exchanged between this package and the background process.
+Must match gpr_query.adb Version.")
+
 ;;;;; sessions
 
 ;; gpr_query reads the project files and the database at startup,
@@ -103,7 +107,6 @@ Value must be alist where each element is \"<name>=<value>\""
 		    'null
 		    (list
 		     (concat "--project=" gpr-file)
-		     (when (not ada-xref-full-path) "--short_file_names")
 		     (when gpr-query--debug-start
 		       (concat "--tracefile=gpr_query.trace")
 		       ;; The file gpr_query.trace should contain: gpr_query=yes
@@ -111,11 +114,19 @@ Value must be alist where each element is \"<name>=<value>\""
       (set-process-query-on-exit-flag (gpr-query--session-process session) nil)
       (gpr-query-session-wait session)
 
-      ;; Check for warnings about invalid directories etc. But some
-      ;; warnings are tolerable, so only abort if process actually
-      ;; died.
       (if (process-live-p (gpr-query--session-process session))
 	  (progn
+	    (goto-char (point-min))
+	    (if (search-forward-regexp "version: \\([0-9]+\\)$")
+		(unless (string-equal (match-string 1) gpr-query-protocol-version)
+		  (user-error "gpr-query version mismatch: elisp %s process %s"
+			      gpr-query-protocol-version
+			      (match-string 1)))
+	      (user-error "gpr-query is an old version; expecting %s" gpr-query-protocol-version))
+
+	    ;; Check for warnings about invalid directories etc. But some
+	    ;; warnings are tolerable, so only abort if process actually
+	    ;; died.
 	    (goto-char (point-min))
 	    (when (search-forward "warning:" nil t)
 	      (if debug-on-error
@@ -265,9 +276,9 @@ Uses `gpr_query'. Returns new list."
 (defconst gpr-query-ident-file-regexp
   ;; C:\Projects\GDS\work_dscovr_release\common\1553\gds-mil_std_1553-utf.ads:252:25
   ;; /Projects/GDS/work_dscovr_release/common/1553/gds-mil_std_1553-utf.ads:252:25
-  ;; gds-mil_std_1553-utf.ads:252:25 - when ada-xref-full-path is nil
-  "\\(\\(?:.:\\\\\\|/\\)?[^:]*\\):\\([0123456789]+\\):\\([0123456789]+\\)"
-  ;; 1                             2                   3
+  ;; gds-mil_std_1553-utf.ads:252:25 - when wisi-xref-full-path is nil
+  "\\(\\(?:.:\\\\\\|/\\)?[^:]*\\):\\([0-9]+\\):\\([0-9]+\\)"
+  ;; 1                              2            3
   "Regexp matching <file>:<line>:<column> where <file> is an absolute file name or basename.")
 
 (defconst gpr-query-ident-file-regexp-alist
@@ -279,17 +290,18 @@ Uses `gpr_query'. Returns new list."
   "Regexp matching <file>:<line>:<column> (<type>)")
 
 (defun gpr-query-compilation (project identifier file line col cmd comp-err &optional local_only append)
-  "Run gpr_query IDENTIFIER:FILE:LINE:COL CMD,
+  "Run gpr_query CMD IDENTIFIER:FILE:LINE:COL,
 with compilation-error-regexp-alist set to COMP-ERR."
   ;; Useful when gpr_query will return a list of references; the user
   ;; can navigate to each result in turn via `next-error'.
 
-  ;; Note that ada-xref-full-path is used in gpr-query--start-process,
-  ;; not here; if it changes, the gpr-query session must be restarted.
-
   ;; Emacs column is 0-indexed, gpr_query is 1-indexed.
-  (let* ((cmd-1 (format "%s %s:%s:%d:%d" cmd identifier file line (1+ col)))
-	 (cmd-2 (if local_only (concat cmd-1 " local_only") cmd-1))
+  (let* ((cmd-1 (concat (format "%s %s:%s:%d:%d"
+				cmd identifier file line (1+ col))
+			(when (member cmd '("refs"))
+			      (if local_only " local_only" " global"))
+			(when (member cmd '("overriding" "overridden_by" "parent_types" "refs"))
+			      (if wisi-xref-full-path " full_file_names" " short_file_names"))))
 	 (session (gpr-query-cached-session project))
 	 (result-count 0)
 	 start-pos prev-content
@@ -302,7 +314,7 @@ with compilation-error-regexp-alist set to COMP-ERR."
 	(forward-line 0)
 	(setq prev-content (buffer-substring (point-min) (point)))))
 
-    (with-current-buffer (gpr-query-session-send session cmd-2 t)
+    (with-current-buffer (gpr-query-session-send session cmd-1 t)
       ;; point is at EOB. gpr_query returns one line per result plus prompt, warnings
       (setq result-count (- (line-number-at-pos) 1))
       (setq start-pos (point-min))
@@ -413,29 +425,23 @@ Enable mode if ARG is positive."
   ;; just enable the menu and keymap
   )
 
-(defun gpr-query--normalize-filename (file &optional dir)
-  "Takes account of filesystem differences."
-  ;; need DIR to handle ada-xref-full-path nil
+(defun gpr-query--normalize-filename (file)
+  "Convert FILE from native format to Emacs standard.
+FILE is from gpr-query."
+  ;; FILE must be abs
   (cond
    ((eq system-type 'windows-nt)
     ;; 'expand-file-name' converts Windows directory
     ;; separators to normal Emacs.  Since Windows file
     ;; system is case insensitive, GNAT and Emacs can
     ;; disagree on the case, so convert all to lowercase.
-    (downcase (expand-file-name file dir)))
+    (downcase (expand-file-name file)))
 
    ((eq system-type 'darwin)
     ;; case-insensitive case-preserving; so just downcase
-    (when (and dir
-	       (not ada-xref-full-path))
-      (setq file (expand-file-name file dir)))
-
     (downcase file))
 
    (t ;; linux
-    (when (and dir
-	       (not ada-xref-full-path))
-      (setq file (expand-file-name file dir)))
     file))
   )
 
@@ -500,6 +506,96 @@ Enable mode if ARG is positive."
     (gpr-query--start-process project session)
     ))
 
+(defun gpr-query-tree-refs (project item op)
+  "Run gpr_query tree command OP on ITEM (an xref-item), return list of xref-items."
+  (with-slots (summary location) item
+    ;; 'location' may have line, column nil
+    (let ((eieio-skip-typecheck t))
+      (with-slots (file line column) location
+	(when (eq ?\" (aref summary 0))
+	  ;; gpr_query wants the quotes stripped
+	  (when column (setq column (+ 1 column)))
+	  (setq summary (substring summary 1 (1- (length summary)))))
+
+	(let ((cmd (format "%s %s:%s:%s:%s full_file_names"
+			   op
+			   summary
+			   (file-name-nondirectory file)
+			   (or line "")
+			   (if column (1+ column) "")))
+	      (result nil)
+	      (session (gpr-query-cached-session project)))
+
+	  (with-current-buffer (gpr-query-session-send session cmd t)
+	    ;; 'gpr_query tree_*' returns a list containing the declarations,
+	    ;; bodies, and references (classwide), in no particular order.
+	    ;;
+	    ;; the format of each line is file:line:column (type)
+	    ;;                            1    2    3       4
+	    ;;
+	    ;; 'type' includes the type name
+
+	    (goto-char (point-min))
+
+	    (while (not (eobp))
+	      (cond
+	       ((looking-at gpr-query-ident-file-type-regexp)
+		;; process line
+		(let* ((found-file (match-string 1))
+		       (found-line (string-to-number (match-string 2)))
+		       (found-col  (1- (string-to-number (match-string 3))))
+		       (found-type (match-string 4))
+		       )
+
+		  (unless found-file
+		    ;; Can be nil if actual file is renamed but gpr-query
+		    ;; database not updated. We abort, rather than just
+		    ;; ignoring this entry, because it means other ref are
+		    ;; probably out of date as well.
+		    (user-error "file '%s' not found; refresh?" (match-string 1)))
+
+		  (setq found-file (gpr-query--normalize-filename found-file))
+
+		  (push (xref-make
+			 (cond
+			  ((string= op "tree_refs")
+			   (if found-type
+			       (if (string-match ";" found-type)
+				   ;; ref is to the identifier
+				   (concat summary " " found-type)
+				 ;; ref is to the controlling type of the identifier
+				 found-type)
+			     summary))
+
+			  ((string= op "tree_defs")
+			   found-type)
+			  )
+			 (xref-make-file-location found-file found-line found-col))
+			result)
+		  ))
+
+	       (t ;; ignore line
+		;;
+		;; This skips GPR_PROJECT_PATH and echoed command at start of buffer.
+		;;
+		;; It also skips warning lines.
+		)
+	       )
+	      (forward-line 1)
+	      )
+
+	    (when (null result)
+	      (user-error "gpr_query did not return any references; refresh?"))
+
+	    (nreverse result) ;; root of tree first.
+	    ))))))
+
+(cl-defmethod wisi-xref-definitions ((_xref gpr-query-xref) project item)
+  (gpr-query-tree-refs project item "tree_defs"))
+
+(cl-defmethod wisi-xref-references ((_xref gpr-query-xref) project item)
+  (gpr-query-tree-refs project item "tree_refs"))
+
 (cl-defmethod wisi-xref-other ((_xref gpr-query-xref) project &key identifier filename line column)
   (when (eq ?\" (aref identifier 0))
     ;; gpr_query wants the quotes stripped
@@ -507,9 +603,18 @@ Enable mode if ARG is positive."
     (setq identifier (substring identifier 1 (1- (length identifier))))
     )
 
+  (let ((temp filename))
+
+    (unless (file-name-absolute-p temp)
+      (setq temp (locate-file filename compilation-search-path)))
+
+    (if temp
+	(setq filename temp)
+      (user-error "'%s' not found in current project - renamed?" filename)))
+
   (setq filename (gpr-query--normalize-filename filename))
 
-  (let ((cmd (format "refs %s:%s:%s:%s"
+  (let ((cmd (format "refs %s:%s:%s:%s global full_file_names"
 		     identifier
 		     (file-name-nondirectory filename)
 		     (or line "")
@@ -543,8 +648,6 @@ Enable mode if ARG is positive."
       ;; Module_Type:/home/Projects/GDS/work_stephe_2/common/1553/gds-hardware-bus_1553-wrapper.ads:171:9 (full declaration)
       ;;
       ;; itc_assert:/home/Projects/GDS/work_stephe_2/common/itc/opsim/itc_dscovr_gdsi/Gds1553/src/Gds1553.cpp:830:9 (reference)
-      ;;
-      ;; if ada-xref-full-path is nil, only the file basename is present.
 
       (message "parsing result ...")
 
@@ -563,7 +666,11 @@ Enable mode if ARG is positive."
 			       most-positive-fixnum))
 		 )
 
-            (setq found-file (gpr-query--normalize-filename found-file (file-name-directory filename)))
+	    (unless found-file
+	      ;; can be nil if actual file is renamed but gpr-query database not updated
+	      (error "file '%s' not found; refresh?" (match-string 1)))
+
+            (setq found-file (gpr-query--normalize-filename found-file))
 
 	    (cond
 	     ((string-equal found-type "declaration")
@@ -578,6 +685,7 @@ Enable mode if ARG is positive."
 	    (when (and (equal found-file filename)
 		       (or
 			(string-equal found-type "body")
+			(string-equal found-type "full declaration")
 			(string-equal found-type "declaration"))
 		       (<= dist min-distance))
 	      ;; The source may have changed since the xref database
@@ -622,8 +730,10 @@ Enable mode if ARG is positive."
 
        ((and
 	 (not line)
-	 (string-equal search-type "body"))
-	;; We started in the body file; find the body
+	 (or
+	  (string-equal search-type "body")
+	  (string-equal search-type "full declaration")))
+	;; We started n the body file; find the body
 	(setq result body-loc))
 
        (decl-loc
@@ -652,7 +762,9 @@ Enable mode if ARG is positive."
     (setq identifier (substring identifier 1 (1- (length identifier))))
     )
 
-  (let ((cmd (format "overridden %s:%s:%d:%d" identifier (file-name-nondirectory filename) line (1+ column)))
+  (let ((cmd (format "overridden %s:%s:%d:%d %s"
+		     identifier (file-name-nondirectory filename) line (1+ column)
+		     (if wisi-xref-full-path "full_file_names" "short_file_names")))
 	(session (gpr-query-cached-session project))
 	result)
     (with-current-buffer (gpr-query-session-send session cmd t)
